<?php
namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ger80908ConsultoriaFiscalController extends \Teknisa\DF\Controllers\TeknisaCrud {
    
    protected $entityManager;
    protected $dataSourceName = 'consultoria_fiscal';
    protected $parameterBagColumns; 
    protected $crudService;
    protected $service;
     
    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function find(Request\Filter  $request, Response $response) {
        parent::find($request, $response);
        $dataSets = $response->getDataSets();
   
        $rows = isset($dataSets[0])? $dataSets[0]->getRows() : $dataSets;
        foreach ($rows as &$row) {
            $row["_hasChild"] = (int) $row["HASCHILD"];
        }
        $dataSet = new DataSet('consultoria_fiscal', $rows);
        $response->setDataSets(array($dataSet));
    }
    
    public function importaRegraSystax(Request $request, Response $response){
        
        $dataSet = $request->getDataSet();
        
        $cenarios = array();
        $produtos = array();
        
        foreach($dataSet->getRows() as $row) {
            if($row["NIVEL"] == 1) {
                if(!isset($cenarios[$row["KEY"]])) {
                    $cenarios[$row["KEY"]] = $row["KEY"];
                }
            }
            else {
                if(!isset($cenarios[$row["PARENT"]])) {
                    if(!isset($produtos[$row["KEY"]])) {
                        $produtos[$row["KEY"]] = array("ID_CENARIO" => $row["PARENT"], "COD_PROD" => $row["COD_PROD"]);
                    }
                }
                
            }
        }
        
        foreach($cenarios as $cenario) {
            $this->importaRegraSystaxCentralizada($cenario, '');
        }
        
        foreach($produtos as $produto) {
            if(!isset($cenarios[$produto["ID_CENARIO"]])) {
                $this->importaRegraSystaxCentralizada($produto["ID_CENARIO"], $produto["COD_PROD"]);
            }
        }
   
    }
    
    private function importaRegraSystaxCentralizada($id_cenario, $cod_prod) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'SYSTAX_IMPPARAM_FISCCENT');
            $procedure->addParam(new Param('P_ID_CENARIO',  Param::PARAM_INPUT,     $id_cenario,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_COD_PROD',    Param::PARAM_INPUT,     $cod_prod,      Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    } 
    
}