<?php
namespace Teknisa\DF\Services;

use Doctrine\ORM\EntityManager;
use Geral\Entities\Paramgeral;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Notafiscal;
use Indeterminado\Entities\Relacnflotenfe;
use Indeterminado\Entities\Cfoppadrao;
use Teknisa\DF\Factories\TransmissaoNfeFactory;

use Teknisa\Libs\Util\Masks\PictureProduto;
use Teknisa\Libs\Util\Masks\Masks;

class ValidacoesTransmissaoNfeService
{
    const ID_STATUS_NF_TRANSMITIDA = 'T';
    const ID_STATUS_NF_APROVADA = 'A';
    const ID_STATUS_NF_EMITIDA = 'E';
    const ID_STATUS_NF_CANCELADA = 'C';
    const ID_SAIDA = 'S';
    const ID_ENTRADA = 'E';
    const ARRAY_DE_ERROS = 'ERROS_TRANSMISSAO_NFE';
    const TIPO_OPERAC_COMPLEMENTO = '03';
    const ID_CONTINGENCIA_SCAN = '3';
    const ID_IMPOSTO_ISSQN = '31';
    const ID_MODELO_DOC_NFE = '55';
    const ID_PAIS_BRASIL_BCO_CENTRAL = '1058';
    const QTD_CHAR_CPF = 11;
    const QTD_CHAR_CNPJ = 14;
    const QTD_CHAR_MUNI_IBGE = 7;
    const VERSAO = 'V_300';
    const REGIME_NORMAL = '3';

    protected $transmissaoNfeFactory;
    protected $entityManager;
    protected $masks;
    private $notasfiscais = array();
    private $modulo;
    private $arrayErros = array();
    private $arrayMsgErros = array(
        'LANC_BLOQ'                  => 'Operação bloqueada. Transmissão sendo realizada por outro usuário',
        'COD_EST_EMI'                => 'Código do IBGE do estado do emitente deve ser cadastrado',
        'NAT_OPE_DEV'                => 'Natureza da operação deve ser cadastrado',
        'SERIE_DEV'                  => 'Código da série de nota fiscal eletrônica deve ser cadastrado',
        'COD_MUN_EMI'                => 'Código do IBGE do município do emitente deve ser cadastrado',
        'SERIE_ORIGEM'               => 'Série da nota fiscal eletrônica para o documento de origem deve ser cadastrada',
        'ISSQN_IM_DEV'               => 'Para operações com tributação de ISSQN a inscrição municipal é obrigatória',
        'CNAE_DEV'                   => 'Código CNAE (Classificação Nacional de Atividades Econômicas) da empresa deve ser cadastrado',
        'COD_PAIS_EMI'               => 'Código do Banco Central do país do emitente deve ser cadastrado',
        'END_EMI_DEV'                => 'Endereço do Emitente deve ser cadastrado',
        'BAI_EMI'                    => 'Bairro do emitente deve ser cadastrado',
        'MUN_EMI'                    => 'Município do emitende deve ser cadastrado',
        'COD_PAIS_DES'               => 'Código do Banco Central do país do destinatário deve ser cadastrado',
        'END_DES_DEV'                => 'Endereço do destinatário deve ser cadastrado',
        'BAI_DES'                    => 'Bairro do destinatário deve ser cadastrado',
        'COD_MUN_DES'                => 'Código IBGE do Município do destinatário deve ser cadastrado',
        'MUN_DES'                    => 'Município do destinatário deve ser cadastrado',
        'IE_TRA_INV'                 => 'IE da transportadora inválido. Inscrição deve conter no mínimo 2 e no máximo 14 caracteres',
        'UF_VEI_TRA'                 => 'UF do veículo da transportadora %s deve ser cadastrado',
        'PLA_VEI_TRA'                => 'Placa do veículo da transportadora deve ser cadastrada',
        'COD_NCM_DEV'                => 'Código NCM (Nomenclatura Comum do Mercosul) do produto %s deve ser informado',
        'COD_NCM_INC'                => 'Código NCM (Nomenclatura Comum do Mercosul) do produto %s inválido. Código deve conter 8 caracteres para um código completo ou 2 caracteres quando informado somente o capítulo',
        'COD_CFOP_DEV'               => 'CFOP para o produto %s deve ser cadastrado',
        'COD_CST_INV'                => 'CST do produto %s inválido. Verificar o primeiro dígito do CST no cadastro do cliente, ou do fornecedor ou da inscrição estadual',
        'DAT_FAB_LOT'                => 'Data de fabricação do lote relacionado ao produto %s deve ser cadastrado',
        'COD_CEAN_INV'               => 'Código EAN do produto %s inválido. Código EAN deve conter 8, 12, 13 ou 14 posições',
        'COD_CSOSN_DEV'              => 'CSOSN (Código de Situação da Operação no Simples Nacional) do produto %s deve ser cadastrado',
        'CST_ICMS_DEV'               => 'CST do ICMS para o produto %s deve ser cadastrado',
        'MOD_BC_DEV'                 => 'Modalidade da Base de Cálculo do ICMS do produto %s deve ser cadastrado',
        'MOD_BCST_DEV'               => 'Modalidade da Base de Cálculo do ICMS do Substituto Tributário do produto %s deve ser cadastrado',
        'CLA_SER_DEV'                => 'Classificação do serviço do produto %s deve ser cadastrada',
        'COD_TRIB_ISSQN_DEV'         => 'Código de Tributação do ISSQN para o produto %s deve ser informado',
        'ICMS_ST_N_RELAC'            => 'Atenção! O imposto ICMS ST do produto %s não contém relacionamento fiscal',
        'ICMS_OU_ISS_DEV'            => 'ICMS ou ISSQN para o Produto %s deve ser cadastrado',
        'CST_IPI_DEV'                => 'CST do IPI para o produto %s deve ser cadastrado',
        'CONFLITO_INCIDENCIA'        => 'Conflito na incidência de impostos para o produto %s. Produtos com ISSQN não poderão ter a incidência de ICMS nem de IPI',
        'CST_PIS_DEV'                => 'CST do PIS para o produto %s deve ser cadastrado',
        'CST_COF_DEV'                => 'CST do COFINS para o produto %s deve ser cadastrado',
        'SEM_FILI_CENT'              => 'Filial não possui centralizadora relacionada',
        'DT_APROV_DEV'               => 'Data de processamento da filial centralizadora deve ser informada',
        'CST_PIS_INCORRETO'          => 'Item %s com Código da Situação Tributária para o imposto PIS inválido',
        'PIS_DEV'                    => 'Produto %s sem incidência de PIS',
        'COF_DEV'                    => 'Produto %s sem incidência de COFINS',
        'MOD_FRETE_DEV'              => 'Modalidade do frete deve ser cadastrado',
        'MOD_FRETE_INV'              => 'Modalidade do frete inválido',
        'TIPO_EMIS_INV'              => 'Tipo de emissão da NF-e inválido',
        'ICMS_N_RELAC'               => 'Atenção! O imposto ICMS do produto %s não contém relacionamento fiscal',
        'ISSQN_N_RELAC'              => 'Atenção! O imposto ISSQN do produto %s não contém relacionamento fiscal',
        'PIS_N_RELAC'                => 'Atenção! O imposto PIS do produto %s não contém relacionamento fiscal',
        'COFINS_N_RELAC'             => 'Atenção! O imposto COFINS do produto %s não contém relacionamento fiscal',
        'CST_COF_INCORRETO'          => 'Item %s com Código da Situação Tributária para o imposto COFINS inválido',
        'CFOP_IMP_INC_IPI'           => 'CFOP do produto %s de Importação e não informado dados do imposto IPI',
        'CFOP_IMP_INC_II'            => 'CFOP do produto %s de Importação e não informado dados do imposto II',
        'INSC_SUFRAMA_DEV'           => 'Inscrição SUFRAMA deve ser informada na venda com isenção para Zona Franca de Manaus',
        'CFOP_INVA_SUFRAMA'          => 'CFOP do produto %s inválido: CFOP de operação isenta para ZFM deve ser 6109 ou 6110',
        'CEP_EMIT_DEV'               => 'CEP do emitente deve ser informado',
        'CEP_EMIT_INC'               => 'CEP do emitente inválido: CEP deve conter 8 caracteres e todos eles devem ser numéricos',
        'CEP_DEST_DEV'               => 'CEP do destinatário deve ser informado',
        'CEP_DEST_INC'               => 'CEP do destinatário inválido: CEP deve conter 8 caracteres e todos eles devem ser numéricos',
        'COD_CFOP_INC'               => 'CFOP do produto %s inválido. Favor informar um CFOP válido',
        'PLA_VEI_TRA_INC'            => 'Placa do veículo da transportadora %s inválido. Formato da placa deve ser XXX9999',
        'DUPLI_MAIOR_120'            => 'Erro na quantidade de duplicatas: O máximo permitido pela SEFAZ são de 120 duplicatas',
        'ADICOES_MAIOR_120'          => 'Erro na quantidade de adições: O máximo permitido pela SEFAZ são de 100 adições por produto',
        'SERIESCAN_DEV'              => 'Código da série de nota fiscal eletrônica (SCAN) deve ser cadastrado',
        'SERIESCAN_ORIGEM'           => 'Série da nota fiscal eletrônica (SCAN) para o documento de origem deve ser cadastrada',
        'CST_IPI_INCORRETO'          => 'Item %s com Código da Situação Tributária para o imposto IPI inválido',
        'ICMSST_N_CAD'               => 'ICMS ST para o Produto %s deve ser cadastrado',
        'COD_IBGE_PROD_RURAL'        => 'Código IBGE do estado do produtor rural da nota %s (CPF/CNPJ: %s, Série: %s) deve ser informado',
        'CDMODDOCFISC_INCORRETO'     => 'Código do modelo do documento fiscal inválido. Utilizar o código 55 para identificação da NF-e',
        'NF_EMIT_APROV'              => 'Operação Bloqueada! Nota fiscal %s sem passar pelo processo eletrônico',
        'COD_CSOSN_INV'              => 'CSOSN (Código de Situação da Operação no Simples Nacional) do produto %s inválido',
        'NFRELACLOTE'                => 'Operação Bloqueada! Nota fiscal relacionada ao lote Empresa: %s Número: %s',
        'CDMUNEMI_INV'               => 'Código do IBGE do município do emitente inválido',
        'CNPJ_CPF_PRODRURAL_INV'     => 'CNPJ/CPF da NF referenciada de produtor inválido',
        'CNPJ_CPF_PRODRURAL_DEVE'    => 'CNPJ/CPF da NF referenciada de produtor deve ser informado',
        'CNPJ_CPF_EMITENTE_INV'      => 'CNPJ/CPF do remetente inválido',
        'CNPJ_CPF_EMITENTE_DEVE'     => 'CNPJ/CPF do remetente deve ser informado',
        'CNPJ_CPF_DESTINATARIO_INV'  => 'CNPJ/CPF do destinatário inválido',
        'CNPJ_CPF_DESTINATARIO_DEVE' => 'CNPJ/CPF do destinatário deve ser informado',
        'CNPJ_CPF_TRANSP_INV'        => 'CNPJ/CPF do transportador inválido',
        'QVOL_DEVE'                  => 'Quantidade de volumes transportados deve ser informado',
        'HORVERAO_DEVE'              => 'Operação Bloqueada! Período do horário de verão deve ser informado',
        'DIFUTCFUSO_DEVE'            => 'Operação Bloqueada! Diferença do fuso horário deve ser informado',
        'DATA_MAIOR'                 => 'Operação Bloqueada! Data de emissão deve ser menor ou igual a data de saída',
        'DTEMISSAO_DIFERE_BD'        => 'Operação Bloqueada! Data de emissão diferente da data corrente %s',
        'VRVENC_DIFERE_VR_NOTA'      => 'Operação Bloqueada! Valor total dos vencimentos difere do valor total da Nota Fiscal',
        'DT_FAB_PROD'                => 'Data de Fabricação do Produto %s não informada. Lote %s'
    );

//     function __construct($notasfiscais, $modulo)
//     {
//         try {
//             $this->notasfiscais = $notasfiscais;
//             $this->modulo       = $modulo;
//         } catch (\Exception $e) {
//             throw $e;
//         }
//     }

    public function __construct(TransmissaoNfeFactory $transmissaoNfeFactory, EntityManager $entityManager)
    {
        $this->transmissaoNfeFactory = $transmissaoNfeFactory;
        $this->entityManager = $entityManager;
        $this->masks = new Masks();
    }

    private function incrementaArrayErros(Notafiscal $notafiscal, $mensagem, $razaosocial){
        $this->arrayErros[] = "\n- Nota {$razaosocial} de valor {$notafiscal->getVrnotafisc()} com erro: {$mensagem}";
        // $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
        //                                   'NRLANCTONF' => $notafiscal->getNrlanctonf(),
        //                                   'MENSAGEM'   => $mensagem,
        //                                   'RAZAOSOCIAL'=> $razaosocial,
        //                                   'VALOR'      => $notafiscal->getVrnotafisc());
    }

    public function validaNFe($notasfiscais, $config)
    {

        /** @var $notafiscal Notafiscal */
        foreach ($notasfiscais as $notafiscal) {
//             if ($this->modulo == self::VERSAO) {
                $corpoNF = self::getCorpoNf($notafiscal);

                /** @var  $paramgeral Paramgeral */
                $paramgeral = $this->entityManager->getRepository('Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $notafiscal->getNrorg()));

                try {
                    $this->transmissaoNfeFactory->consultaBloqueiaNotaFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
                } catch (\Exception $e) {
                    $mensagem = $this->arrayMsgErros['LANC_BLOQ'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }

                // try{
                //     $iderroaviso       = $this->transmissaoNfeFactory->getIdErroAviso($notafiscal->getCdfilial());
                //     $valorTotalImposto = $this->transmissaoNfeFactory->getValorTotalImposto(
                //         $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $iderroaviso[0]["IDERROAVISOFIL"], $paramgeral->getNrdecitotanf() ? : 2
                //     );
                //     $valorSubtracaoVencimentos = $this->transmissaoNfeFactory->getValorSubtracaoVencimentos(
                //         $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getIdentrsaidop(), $notafiscal->getCdtipooper()
                //     );

                //     $valorTotalVencimentos = $this->transmissaoNfeFactory->getValorTotalVencimentos(
                //         $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $valorTotalImposto[0]["V_VRIMPOSTOT"], $valorSubtracaoVencimentos[0]["VRSUBTRAIVENC"]
                //     );

                //     if($valorTotalVencimentos[0]["VRDIFERENCA"] != 0)
                //         throw new \Exception();
                // }catch (\Exception $e){
                //     $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                //         'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                //         'MENSAGEM'   => $this->arrayMsgErros['VRVENC_DIFERE_VR_NOTA']);
                // }

                $relacnflotenfe = $this->entityManager->getRepository('Indeterminado\Entities\Relacnflotenfe')->findOneBy(
                    array('cdfilial' => $notafiscal->getCdfilial(),
                            'nrlanctonf' => $notafiscal->getNrlanctonf(),
                            'idstatusrelnfe' => 'T',
                            'nrorg' => $notafiscal->getNrorg()));

                if ($relacnflotenfe instanceof Relacnflotenfe) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['NFRELACLOTE'], array($notafiscal->getCdemplotenfe(), $notafiscal->getNrlotenfe()));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }

                $consisteFiscal = $this->transmissaoNfeFactory->consisteFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
                if (is_array($consisteFiscal) && !empty($consisteFiscal)) {
                    foreach ($consisteFiscal as $erro) {
                        if (empty($erro['CDIMPOSTO'])) {
                            $mensagem = $erro['DSCONSISTENCIA'] . ' (Produto: ' . $erro['CDARVPROD'];
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        } else {
                            $mensagem = $erro['DSCONSISTENCIA'] . ' (Produto: ' . $erro['CDARVPROD'] . ' - Imposto: ' . $erro['CDIMPOSTO'];
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                    }
                }

                switch ($notafiscal->getIdstatus()) {
                case self::ID_STATUS_NF_APROVADA:
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['NF_EMIT_APROV'], array('aprovada'));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    break;
                case self::ID_STATUS_NF_EMITIDA:
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['NF_EMIT_APROV'], array('emitida'));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    break;
                case self::ID_STATUS_NF_CANCELADA:
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['NF_EMIT_APROV'], array('cancelada'));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }

                if (empty($corpoNF['CDESTAIBGEEMIT'])) {
                    $mensagem = $this->arrayMsgErros['COD_EST_EMI'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
                $e_dscdfisopenf = $notafiscal->getDscdfisopenf();
                
                if (empty($e_dscdfisopenf)) {
                    $mensagem = $this->arrayMsgErros['NAT_OPE_DEV'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
                $e_cdserie = $corpoNF['CDSERIE'];
                if (empty($e_cdserie) && $e_cdserie != '0') {
                    $mensagem = $this->arrayMsgErros['SERIE_DEV'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }

                if ($paramgeral->getIdutdtproctnfe() == 'S') {
                    $filiOrig = $this->transmissaoNfeFactory->consultaFilialOrigem($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
                    $filiCent = $this->transmissaoNfeFactory->consultaFilialCentralizadora($filiOrig['CDFILIORIG'], $notafiscal->getNrorg());

                    if (empty($filiCent['CDFILIFISC'])) {
                        $mensagem = $this->arrayMsgErros['SEM_FILI_CENT'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    } elseif (empty($filiCent['DTPROCESSA']) || $filiCent['DTPROCESSA'] == 0) {
                        $mensagem = $this->arrayMsgErros['DT_APROV_DEV'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                }

                if ($paramgeral->getIddtemisdifdtbc() == 'S') {
                    if (!DateUtil::comparaDatas(DateUtil::getDataDeString($corpoNF['DTEMISSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA), DateUtil::getDataAtual(true), DateUtil::IGUAL)) {
                        $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['DTEMISSAO_DIFERE_BD'], array(DateUtil::getDataAtual(true)));
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                }

                //Em caso de nota de devolução
                if ($corpoNF['IDOPERDEVONF'] == 'S' || $corpoNF['CDTIPOOPER'] == self::TIPO_OPERAC_COMPLEMENTO || $corpoNF['IDNFESTCANC'] == 'S') {
                    $notaDevolucao = $this->transmissaoNfeFactory->consultaNotaDevolucao($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());

                    if (is_array($notaDevolucao) && !empty($notaDevolucao)) {
                        foreach ($notaDevolucao as $erro) {
                            if (empty($erro['NRACESSONFE']) && empty($erro['CDSERIE'])) {
                                $mensagem = $this->arrayMsgErros['SERIE_ORIGEM'];
                                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                            }
                        }
                    }
                }

                $verIncImposNFe = $this->transmissaoNfeFactory->consultaItemProdutoImpostoNFE($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), self::ID_IMPOSTO_ISSQN, $notafiscal->getNrorg());
                if (is_array($verIncImposNFe) && !empty($verIncImposNFe)) {
                    if (empty($corpoNF['CDINSCMUNI'])) {
                        $mensagem = $this->arrayMsgErros['ISSQN_IM_DEV'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    } elseif (empty($corpoNF['CDCNAE'])) {
                        $mensagem = $this->arrayMsgErros['CNAE_DEV'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                }

                if (empty($corpoNF['CDMODDOCFISC']) || $corpoNF['CDMODDOCFISC'] != self::ID_MODELO_DOC_NFE) {
                    $mensagem = $this->arrayMsgErros['CDMODDOCFISC_INCORRETO'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
                if ($paramgeral->getIdutdtbcdtemiss() == 'N' && $paramgeral->getIdutdtproctnfe() == 'S') {
                    if (DateUtil::comparaDatas(DateUtil::getDataDeString($corpoNF['DTENTRSAID'], DateUtil::FORMATO_BRASILEIRO_DATAHORA), DateUtil::getDataDeString($corpoNF['DTEMISSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA))) {
                        $mensagem = $this->arrayMsgErros['DATA_MAIOR'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                }

                $this->validaEmitente($corpoNF, $notafiscal);
                $this->validaDestinatario($corpoNF, $notafiscal);
                $this->validaProdutorRural($corpoNF, $notafiscal);
                $this->validaItens($corpoNF, $notafiscal, $config->regime);
                $this->verificaDuplicata($corpoNF, $notafiscal);
                //$this->validaTransportadora($corpoNF, $notafiscal);
//             }
        }

        if(!empty($this->arrayErros))
            $this->arrayErros[0] = trim($this->arrayErros[0]);
        return $this->arrayErros;
    }

    private function getCorpoNf(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        $cdfilial = $notafiscal->getCdfilial();

        /* @@var \Fiscal\Repositories\Notafiscal */
        return $this->transmissaoNfeFactory->getCorpoNotaFiscal(
            $cdfilial,
            $notafiscal->getNrlanctonf(),
            $notafiscal->getIdorigdestnf(),
            $notafiscal->getIdentrsaidop(),
            $notafiscal->getNrorg()
        );
    }

    private static function adicionaParametroMsgErro($mensagem, $param = array())
    {
        foreach ($param as $replace) {
            $mensagem = substr_replace($mensagem, $replace, strpos($mensagem, '%s'), 2);
        }
        return $mensagem;
    }

    /**
     * @param $corpoNF
     * @param $notafiscal
     */
    private function validaEmitente($corpoNF, Notafiscal $notafiscal)
    {
        if ($corpoNF['CDPAISBACENEMIT'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            $e_nrinsjuremit = trim($this->masks->removeMask($corpoNF['NRINSJUREMIT'], "/[^A-Za-z0-9 ]/"));
            if (!empty($e_nrinsjuremit)) {
                if (!$this->validaInscricao(trim($this->masks->removeMask($corpoNF['NRINSJUREMIT'], "/[^A-Za-z0-9 ]/")))) {
                    $mensagem = $this->arrayMsgErros['CNPJ_CPF_EMITENTE_INV'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            } else {
                $mensagem = $this->arrayMsgErros['CNPJ_CPF_EMITENTE_DEVE'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }

        if (empty($corpoNF['CDPAISBACENEMIT'])) {
            $mensagem = $this->arrayMsgErros['COD_PAIS_EMI'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }

        if (empty($corpoNF['DSENDEEMIT'])) {
            $mensagem = $this->arrayMsgErros['END_EMI_DEV'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }

        if (empty($corpoNF['NMBAIREMIT'])) {
            $mensagem = $this->arrayMsgErros['BAI_EMI'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }

        if ($corpoNF['CDPAISBACENEMIT'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            if (empty($corpoNF['CDMUNIIBGEEMIT'])) {
                $mensagem = $this->arrayMsgErros['COD_MUN_EMI'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif (strlen(trim($this->masks->removeMask($corpoNF['CDMUNIIBGEEMIT'], "/[^A-Za-z0-9 ]/"))) != 7) {
                if (strlen(trim($corpoNF['CDESTAIBGEEMIT']) . trim($corpoNF['CDMUNIIBGEEMIT'])) != 7) {
                    $mensagem = $this->arrayMsgErros['CDMUNEMI_INV'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }

            if (empty($corpoNF['NMMUNIEMIT'])) {
                $mensagem = $this->arrayMsgErros['MUN_EMI'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }
        $e_nrcepemit = $this->masks->removeMask($corpoNF['NRCEPEMIT'], "/[^A-Za-z0-9 ]/");
        if (empty($e_nrcepemit)) {
            $mensagem = $this->arrayMsgErros['CEP_EMIT_DEV'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        } elseif (!is_numeric($this->masks->removeMask($corpoNF['NRCEPEMIT'], "/[^A-Za-z0-9 ]/"))) {
            $mensagem = $this->arrayMsgErros['CEP_EMIT_INC'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }
    }

    public function validaInscricao($inscricao)
    {
        switch (strlen($inscricao)) {
        case(self::QTD_CHAR_CPF): {
            return ($this->validaCPF($inscricao) == true);
            break;
        }
        case(self::QTD_CHAR_CNPJ): {
            return ($this->masks->validaCNPJ($inscricao) == true);
            break;
        }
        default:
            return false;
        }
    }

    /**
     * @param $notafiscal
     * @param $corpoNF
     */
    private function validaDestinatario($corpoNF, Notafiscal $notafiscal)
    {
        $cupomFiscal = $this->transmissaoNfeFactory->consultaCupomFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
        if (is_array($cupomFiscal) && !empty($cupomFiscal)) {
            $cupomFiscal = $cupomFiscal[0];
            if ($cupomFiscal['CDPAISBACEN'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
                if (trim($this->masks->removeMask($cupomFiscal['NRINSJUDESNF'], "/[^A-Za-z0-9 ]/"))) {
                    if (!$this->validaInscricao(trim($this->masks->removeMask($cupomFiscal['NRINSJUDESNF'], "/[^A-Za-z0-9 ]/")))) {
                        $mensagem = $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_INV'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                } else {
                    $mensagem = $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_DEVE'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
        } elseif ($corpoNF['CDPAISBACENCLIE'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            if (trim($this->masks->removeMask($corpoNF['NRINSJURCLIE'], "/[^A-Za-z0-9 ]/"))) {
                if (!$this->validaInscricao(trim($this->masks->removeMask($corpoNF['NRINSJURCLIE'], "/[^A-Za-z0-9 ]/")))) {
                    $mensagem = $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_INV'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            } else {
                $mensagem = $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_DEVE'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }

        if (empty($corpoNF['CDPAISBACENCLIE'])) {
            $mensagem = $this->arrayMsgErros['COD_PAIS_DES'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }
        if (empty($corpoNF['DSENDECLIE'])) {
            $mensagem = $this->arrayMsgErros['END_DES_DEV'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }
        if (empty($corpoNF['NMBAIRCLIE'])) {
            $mensagem = $this->arrayMsgErros['BAI_DES'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }
        if ($corpoNF['CDPAISBACENCLIE'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            if (empty($corpoNF['CDMUNIIBGECLIE'])) {
                $mensagem = $this->arrayMsgErros['COD_MUN_DES'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif (strlen(trim($this->masks->removeMask($corpoNF['CDMUNIIBGECLIE'], "/[^A-Za-z0-9 ]/"))) != 7 && strlen(trim($this->masks->removeMask($corpoNF['CDESTAIBGECLIE'] . $corpoNF['CDMUNIIBGECLIE'], "/[^A-Za-z0-9 ]/"))) != 7) {
                $mensagem = $this->arrayMsgErros['COD_MUN_DES'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            if (empty($corpoNF['NMMUNICLIE'])) {
                $mensagem = $this->arrayMsgErros['MUN_DES'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }
        $e_nrcepclie = $this->masks->removeMask($corpoNF['NRCEPCLIE'], "/[^A-Za-z0-9 ]/");
        if (empty($e_nrcepclie) && $corpoNF['CDPAISBACENCLIE'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            $mensagem = $this->arrayMsgErros['CEP_DEST_DEV'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        } elseif (!is_numeric($this->masks->removeMask($corpoNF['NRCEPCLIE'], "/[^A-Za-z0-9 ]/")) && $corpoNF['CDPAISBACENCLIE'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            $mensagem = $this->arrayMsgErros['CEP_DEST_INC'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }
    }

    /**
     * @param $corpoNF
     * @param $notafiscal
     */
     /*
    private function validaTransportadora($corpoNF, Notafiscal $notafiscal)
    {
        $e_nrinsjurtran = trim($this->masks->removeMask($corpoNF['NRINSJURTRAN'], "/[^A-Za-z0-9 ]/"));
        if (!empty($e_nrinsjurtran)) {
            if (!$this->validaInscricao(trim($this->masks->removeMask($corpoNF['NRINSJURTRAN'], "/[^A-Za-z0-9 ]/")))) {
                $mensagem = $this->arrayMsgErros['CNPJ_CPF_TRANSP_INV'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            if ($corpoNF['NRINSESTTRAN'] && !((strlen($this->masks->removeMask($corpoNF['NRINSESTTRAN'], "/[^A-Za-z0-9 ]/")) >= 2) && (strlen($this->masks->removeMask($corpoNF['NRINSESTTRAN'], "/[^A-Za-z0-9 ]/")) <= 14))) {
                $mensagem = $this->arrayMsgErros['IE_TRA_INV'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            if ($corpoNF['IDPGTOFRETE'] == '') {
                $mensagem = $this->arrayMsgErros['MOD_FRETE_DEV'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif ($corpoNF['IDPGTOFRETE'] && !in_array($corpoNF['IDPGTOFRETE'], array('0', '1', '2', '9'))) {
                $mensagem = $this->arrayMsgErros['MOD_FRETE_INV'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            $e_nrplacaveic = $this->masks->removeMask($corpoNF['NRPLACAVEIC'], "/[^A-Za-z0-9 ]/");
            if (!empty($e_nrplacaveic) || $corpoNF['SGUFVEIC']){
                $this->verificaPlacaVeiculo($corpoNF, $notafiscal);

                if (empty($corpoNF['SGUFVEIC'])) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['UF_VEI_TRA'], array($corpoNF['NMRAZSOCTRAN']));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
        }
    }*/

    /**
     * @param            $corpoNF
     * @param Notafiscal $notafiscal
     */
    private function verificaPlacaVeiculo($corpoNF, Notafiscal $notafiscal)
    {
        $e_nrplacaveic = $this->masks->removeMask($corpoNF['NRPLACAVEIC'], "/[^A-Za-z0-9 ]/");
        if (empty($e_nrplacaveic)) {
            $mensagem = $this->arrayMsgErros['PLA_VEI_TRA'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        } else {
            $erroPlaca = false;
            $stNrPlaca = $this->masks->removeMask($corpoNF['NRPLACAVEIC'], "/[^A-Za-z0-9 ]/");
            for ($i = 0; $i < strlen($stNrPlaca); $i++) {
                if ($i <= 2) {
                    if (!ctype_alpha($stNrPlaca[$i])) {
                        $erroPlaca = true;
                        break;
                    }
                } else {
                    if (!is_numeric($stNrPlaca[$i])) {
                        $erroPlaca = true;
                        break;
                    }
                }
            }
            if ($erroPlaca) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['PLA_VEI_TRA_INC'], array($corpoNF['NMRAZSOCTRAN']));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }
    }

    /**
     * @param            $corpoNF
     * @param Notafiscal $notafiscal
     */
    private function validaProdutorRural($corpoNF, Notafiscal $notafiscal)
    {
        $nfProdRural = $this->transmissaoNfeFactory->consultaNotaFiscalProdRural($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());

        if (is_array($nfProdRural) && !empty($nfProdRural)) {
            foreach ($nfProdRural as $prodRural) {
                $e_nrinsjuriest = trim($this->masks->removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/"));
                if (!empty($e_nrinsjuriest)) {
                    if (!self::validaInscricao(trim($this->masks->removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/")))) {
                        $mensagem = $this->arrayMsgErros['CNPJ_CPF_PRODRURAL_INV'];
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                } else {
                    $mensagem = $this->arrayMsgErros['CNPJ_CPF_PRODRURAL_DEVE'];
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
                if (strlen($this->masks->removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/") == 14)) {
                    $stNrInsProdRural = $this->masks->mask($this->masks->removeMask(trim($prodRural['NRINSJURIEST']), "/[^A-Za-z0-9 ]/"), $this->masks->picturePessoa('J'));
                } else {
                    if (strlen($this->masks->removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/") == 11)) {
                        $stNrInsProdRural = $this->masks->mask($this->masks->removeMask(trim($prodRural['NRINSJURIEST']), "/[^A-Za-z0-9 ]/"), $this->masks->picturePessoa('F'));
                    } else {
                        $stNrInsProdRural = $prodRural['NRINSJURIEST'];
                    }
                }
                if (empty($prodRural['CDESTADOIBGE'])) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_IBGE_PROD_RURAL'], array($prodRural['NRNOTAFISC'], $stNrInsProdRural, $prodRural['CDSERIE']));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
        }
    }

    private function validaItens($corpoNF, Notafiscal $notafiscal, $cdsitucrt)
    {
        $itemNota = $this->transmissaoNfeFactory->consultaItemNotaFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $corpoNF['CDPAISCLIE'], $corpoNF['SGESTACLIE'], $notafiscal->getNrorg());

        foreach ($itemNota as $item) {
            if (empty($item['NCM'])) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_NCM_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif (!(strlen($this->masks->removeMask($item['NCM'], "/[^A-Za-z0-9 ]/")) == 2) ) {
                if (!(strlen($this->masks->removeMask($item['NCM'], "/[^A-Za-z0-9 ]/")) == 8) ) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_NCM_INC'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
            if (empty($item['CFOP'])) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CFOP_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif ($this->validaCFOPpadrao($item['CFOP']) != 'S') {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CFOP_INC'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            if ($item['CDSITUTRIBPR']) {
                if (strlen($item['CDSITUTRIBPR']) != 3) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CST_INV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            } else {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CST_INV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            if ($item['IDSEGMENTOPROD'] == 'MEDI' && empty($item['DTFABRPROD'])) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['DAT_FAB_LOT'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            if ($item['CDBARPRODUTO']) {
                if(!(strlen($item['CDBARPRODUTO']) <= 7)){
                    if ( !(strlen($item['CDBARPRODUTO']) == 8) && !(strlen($item['CDBARPRODUTO']) == 12) && !(strlen($item['CDBARPRODUTO']) == 13) && !(strlen($item['CDBARPRODUTO']) == 14)) {
                        $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CEAN_INV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                }
            }
            if ($cdsitucrt == '1') {
                if (empty($item['CDFISCOPERSN'])) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CSOSN_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
                if ($item['CSOSN'] && strlen($item['CSOSN']) != 4) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CSOSN_INV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }

            $verifAdicoes = $this->transmissaoNfeFactory->verificaAdicoes($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $item['NRSEQITPROD'], $notafiscal->getNrorg());
            if ($verifAdicoes > 100) {
                $mensagem = $this->arrayMsgErros['ADICOES_MAIOR_120'];
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
            
            $prodFili = $this->transmissaoNfeFactory->getProdFiliTrans($notafiscal->getCdfiliemit(), $item["CDPRODUTO"]);
            if($prodFili) {
                if($prodFili["IDCNTRESTOQ"] == 'S' && $prodFili["IDUTCTRLLOTE"] == 'S' && $prodFili["IDPRODCOMRASTRO"] =='S') {
                    if(!$item["DTFABRPROD"]) {
                        $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['DT_FAB_PROD'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg()), $item["NRLOTEESTQ"]));
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                }
            }
            
            $this->validaImpostosItens($corpoNF, $notafiscal, $item, $cdsitucrt);
        }
    }

    private function maskProduto($cdarvprod, $nrorg)
    {
        return $this->masks->mask($cdarvprod, preg_replace("/([^.])/", '#', PictureProduto::mascaraProduto($nrorg)));
    }



    private function validaImpostosItens($corpoNF, Notafiscal $notafiscal, $item, $cdsitucrt)
    {
        $impostosItem = $this->transmissaoNfeFactory->consultaImpostosItem($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRSEQITPROD'], $item['NRORG']);
        $impostosNFE  = $this->transmissaoNfeFactory->retornaImpostosNFE($item['NRORG']);

        $impostosNfeICMS = array_filter($impostosNFE, function($aImposto){
            return $aImposto["CDIMPOSTONFE"] == "00";});
        $impostosNfeICMS = end($impostosNfeICMS);

        if (empty($impostosNfeICMS)) {
            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['ICMS_N_RELAC'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        } else {
            $impostosItemICMS = array_filter($impostosItem, function($imposto){
                return $imposto["CDIMPOSTONFE"] == "00";});
            $impostosItemICMS = end($impostosItemICMS);

            if (!empty($impostosItemICMS)) {
                //Só valida o CST do ICMS caso o Código do Regime Tributário da empresa seja "3 - Regime Normal"
                if ($cdsitucrt == self::REGIME_NORMAL) {
                    if (empty($impostosItemICMS['CST'])) {
                        $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_ICMS_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                    if ($impostosItemICMS['CST'] == '00' || $impostosItemICMS['CST'] == '10' || $impostosItemICMS['CST'] == '20' || $impostosItemICMS['CST'] == '30'
                        || $impostosItemICMS['CST'] == '51' || $impostosItemICMS['CST'] == '70' || $impostosItemICMS['CST'] == '90'
                    ) {
                        if ($impostosItemICMS['CST'] != '30' && empty($impostosItemICMS['IDMODBCITNFE'])) {
                            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['MOD_BC_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                        if ($impostosItemICMS['IDMODBCSTITNFE'] == '10' || $impostosItemICMS['IDMODBCSTITNFE'] == '30' || $impostosItemICMS['IDMODBCSTITNFE'] == '70' || $impostosItemICMS['IDMODBCSTITNFE'] == '90') {
                            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['MOD_BCST_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                    } elseif ($impostosItemICMS['CST'] != '40' || $impostosItemICMS['CST'] != '41' || $impostosItemICMS['CST'] != '50') {
                        if ($impostosItemICMS['IDIMPDESONERAIT'] == 'S' && $impostosItemICMS['IDMOTDESONERAIT'] == '7') {
                            if (empty($corpoNF['NRINSCSUFRAMA'])) {
                                $mensagem = $this->arrayMsgErros['INSC_SUFRAMA_DEV'];
                                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                            }
                            if (!$item['CFOP'] == '6109' || !$item['CFOP'] == '6110') {
                                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CFOP_INVA_SUFRAMA'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                            }
                        }
                    }
                } else {
                    //Se ele for simples e o CSOSN usar BC e tiver ICMS valida a BC do ICMS.
                    if ($item['CDFISCOPERSN'] == '201' || $item['CDFISCOPERSN'] == '202' || $item['CDFISCOPERSN'] == '203' || $item['CDFISCOPERSN'] == '900') {
                        if (empty($impostosItemICMS['IDMODBCITNFE'])) {
                            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['MOD_BCST_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                    }
                }
            } else {
                //Imposto ISSQN
                $impostosNfeISSQN = array_filter($impostosNFE, function($imposto){
                    return $imposto["CDIMPOSTONFE"] == "31";});
                $impostosNfeISSQN = end($impostosNfeISSQN);

                // Foi verificado que existe ISSQN no simples também, logo as validações abaixo serão validas.
                if (empty($impostosNfeISSQN)) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['ISSQN_N_RELAC'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                } else {
                    $impostosItemISSQN = array_filter($impostosItem, function($imposto){
                        return $imposto["CDIMPOSTONFE"] == "31";});
                    $impostosItemISSQN = end($impostosItemISSQN);

                    if (!empty($impostosItemISSQN)) {
                        //Verifica o codigo para classificação do serviço
                        $classificacaoProduto = $this->transmissaoNfeFactory->consultaClassificacaoServico($item['CDARVPROD']);

                        if (empty($classificacaoProduto['CDCLASSPROD'])) {
                            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CLA_SER_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                        if (empty($classificacaoProduto['CDTRIBISSQN'])) {
                            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COD_TRIB_ISSQN_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                        }
                    } else {
                        $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['ICMS_OU_ISS_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                        $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                    }
                }
            }
        }

        //Imposto IPI
        $impostosItemIPI = array_filter($impostosItem, function($imposto) {return $imposto["CDIMPOSTONFE"] == "10"; });
        $impostosItemIPI = end($impostosItemIPI);

        if (!empty($impostosItemIPI)) {
            //Só valida o CST do IPI caso esteja vazio e o Código do Regime Tributário da empresa seja "3 - Regime Normal"
            if ($cdsitucrt == '3') {
                if (empty($impostosItemIPI['CST'])) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_IPI_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                } elseif (!in_array($impostosItemIPI['CST'], array('00', '01', '02', '03', '04', '05', '49', '50', '51', '52', '53', '54', '55', '99'))) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_IPI_INCORRETO'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
        } else {
            if ($item['CFOP'] && $item['CFOP'] == '3') {
                if ($item['CFOP'] != '3201' || $item['CFOP'] != '3202' || $item['CFOP'] != '3211' || $item['CFOP'] != '3503' || $item['CFOP'] != '3553') {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CFOP_IMP_INC_IPI'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
        }

        // Verificando se existe incidência de ICMS, IPI e ISSQN para o mesmo produto.
        if (!empty($impostosItemISSQN)) {
            if (!empty($impostosItemICMS) || !empty($impostosItemIPI)) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CONFLITO_INCIDENCIA'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }
        if ($item['CFOP'] && $item['CFOP'] == '3') {
            if ($item['CFOP'] != '3201' || $item['CFOP'] != '3202' || $item['CFOP'] != '3211' || $item['CFOP'] != '3503' || $item['CFOP'] != '3553') {
                //Imposto II
                $impostosItemII = array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "20";'));
                $impostosItemII = end($impostosItemII);

                if (empty($impostosItemII)) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CFOP_IMP_INC_II'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            }
        }

        //Imposto PIS
        $impostosNfePIS = array_filter($impostosNFE, 
                                        function($imposto) { 
                                            return $imposto["CDIMPOSTONFE"] == "50";
                                        });
        $impostosNfePIS = end($impostosNfePIS);

        if (empty($impostosNfePIS)) {
            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['PIS_N_RELAC'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        } else {
            $impostosItemPIS = array_filter($impostosItem, function($imposto) { 
                                return $imposto["CDIMPOSTONFE"] == "50";
            });
            $impostosItemPIS = end($impostosItemPIS);

            if (empty($impostosItemPIS)) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['PIS_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif ($impostosItemPIS['CST']) {
                if (!in_array(
                    $impostosItemPIS['CST'], array('01', '02', '03', '04', '05', '06', '07', '08', '09', '49', '50', '51', '52', '53', '54', '55', '56',
                        '60', '61', '62', '63', '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99')
                )
                ) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_PIS_INCORRETO'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            } else {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_PIS_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }

        //Imposto COFINS
        $impostosNfeCOFINS = array_filter($impostosNFE, function($imposto) {
                                return $imposto["CDIMPOSTONFE"] == "60";
                            });
        $impostosNfeCOFINS = end($impostosNfeCOFINS);

        if (empty($impostosNfeCOFINS)) {
            $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COFINS_N_RELAC'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        } else {
            $impostosItemCOFINS = array_filter($impostosItem, function($imposto) {
                                    return $imposto["CDIMPOSTONFE"] == "60";
            });
            $impostosItemCOFINS = end($impostosItemCOFINS);

            if (empty($impostosItemCOFINS)) {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['COF_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            } elseif ($impostosItemCOFINS['CST']) {
                if (!in_array(
                    $impostosItemCOFINS['CST'], array('01', '02', '03', '04', '05', '06', '07', '08', '09', '49', '50', '51', '52', '53', '54', '55', '56',
                        '60', '61', '62', '63', '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99')
                )
                ) {
                    $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_COF_INCORRETO'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                    $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
                }
            } else {
                $mensagem = self::adicionaParametroMsgErro($this->arrayMsgErros['CST_COF_DEV'], array($this->maskProduto($item['CDARVPROD'], $notafiscal->getNrorg())));
                $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
            }
        }
    }

    private function verificaDuplicata($corpoNF, Notafiscal $notafiscal)
    {
        $duplicata = $this->transmissaoNfeFactory->consultaQuantidadeDuplicata($corpoNF['CDFILIAL'], $corpoNF['NRLANCTONF'], $corpoNF['NRORG']);

        if ($duplicata[0]['QTDDUPLICATA'] > 120) {
            $mensagem = $this->arrayMsgErros['DUPLI_MAIOR_120'];
            $this->incrementaArrayErros($notafiscal, $mensagem, $corpoNF['NMRAZSOCCLIE']);
        }
    }

    public function validaCFOPpadrao($cdcfoppadrao)
    {
        $cfoppadrao = $this->entityManager->getRepository('Indeterminado\Entities\Cfoppadrao')->findOneBy(array('cdcfoppadrao' => $cdcfoppadrao));
        if (is_object($cfoppadrao)) {
            return 'S';
        }
        return 'N';
    }
    
    //TODO: Retornar a utilizar a função presente em Teknisa\Libs\Util\Masks\Masks
    private function validaCPF($cpf){
        $dv = null;
        $cpf = trim($this->removeCharsFromString($cpf));
        if(empty($cpf) || strlen($cpf) != 11) {
            return false;
        }else{
            if(!$this->validaInscricaoNumerosIguais($cpf, 11)) {
                return false;
            }else{
                $sub_cpf = substr($cpf, 0, 9);
                for($i = 0; $i <= 9; $i++) {
                    $dv += (intval(substr($sub_cpf,$i,1)) * (10-$i));
                }
                if($dv == 0) {
                    return false;
                }
                $dv = 11 - ($dv % 11);
                if($dv > 9){
                    $dv = 0;
                }
                //if($cpf[9] != $dv) {
                //return false;
                //}
                if(substr($cpf,9,1) != $dv) {
                    return false;
                }
                $dv *= 2;
                for($i = 0; $i <= 9; $i++) {
                    $dv += (intval(substr($sub_cpf,$i,1)) * (11-$i));
                }
                $dv = 11 - ($dv % 11);
                if($dv > 9) {
                    $dv = 0;
                }
                if(substr($cpf,10,1) != $dv) {
                    return false;
                }
                return true;
            }
        }
    }
    
    //TODO: Retornar a utilizar a função presente em Teknisa\Libs\Util\Masks\Masks
    private function removeCharsFromString($string) {
        return preg_replace('/[^0-9]/', '', $string);
    }
    
    //TODO: Retornar a utilizar a função presente em Teknisa\Libs\Util\Masks\Masks
    private function validaInscricaoNumerosIguais($string, $length) {
        for($i = 0; $i <= 9; $i++) {
            $fake = str_pad("", $length, $i);
            if($string === $fake)
                return false;
        }
        return true;
    }
}